from django.db import models
from django.utils.translation import gettext as _

from aleksis.core.mixins import ExtensibleModel
from aleksis.core.models import Person


class CardPrinterStatus(models.TextChoices):
    ONLINE = "online", _("Online")
    OFFLINE = "offline", _("Offline")
    WITH_ERRORS = "with_errors", _("With errors")


class PrintStatus(models.TextChoices):
    REGISTERED = "registered", _("Registered")
    IN_PROGRESS = "in_progress", _("In progress")
    FINISHED = "finished", _("Finished")


class CardPrinter(ExtensibleModel):
    name = models.CharField(max_length=255, verbose_name=_("Name"))
    description = models.TextField(verbose_name=_("Description"), blank=True)
    location = models.CharField(max_length=255, verbose_name=_("Location"))

    status = models.CharField(
        max_length=255, verbose_name=_("Status"), choices=CardPrinterStatus.choices
    )
    status_text = models.TextField(verbose_name=_("Status text"), blank=True)
    last_seen_at = models.DateTimeField(verbose_name=_("Last seen at"), blank=True, null=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("Card printer")
        verbose_name_plural = _("Card printers")


class Card(ExtensibleModel):
    person = models.ForeignKey(
        Person, models.CASCADE, verbose_name=_("Person"), related_name="cards"
    )
    chip_number = models.IntegerField(verbose_name=_("Chip Number"))
    valid_until = models.DateField(verbose_name=_("Valid until"))
    deactivated = models.BooleanField(verbose_name=_("Deactivated"), default=False)

    # Print status
    printed_with = models.ForeignKey(
        CardPrinter,
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        verbose_name=_("Printed with"),
    )
    print_started_at = models.DateTimeField(verbose_name=_("Printed at"), blank=True, null=True)
    print_finished_at = models.DateTimeField(verbose_name=_("Printed at"), blank=True, null=True)

    @property
    def print_status(self) -> PrintStatus:
        if self.print_finished_at:
            return PrintStatus.FINISHED
        elif self.print_started_at:
            return PrintStatus.IN_PROGRESS
        else:
            return PrintStatus.REGISTERED

    class Meta:
        verbose_name = _("Card")
        verbose_name_plural = _("Cards")
